/*
    Copyright (c) 2002 Gregory Montoir

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sdlsms.h"
#include "saves.h"
#include "config.h"
#include "shared.h"

#define SDLVERSION 2



config cfg = { 
  22050, /* sound rate (0 for no sound) */
  1, /* throttle enabled */
  0, /* state_slot to use */
  1, /* 1 for no frame skip */
  0, /* start windowed (not fullscreen) */
  0, /* do not use sram (save, load) before exiting */
  ""};



void parse_args(int argc, char **argv)
{
  int i;
  for(i=2; i<argc; ++i) {
	if(strcasecmp(argv[i], "--fskip") == 0) {
	  if(++i<argc) {
		cfg.frame_skip = atoi(argv[i]);
		cfg.throttle = 0;
	  }
	}
	else if(strcasecmp(argv[i], "--fullspeed") == 0) {
	  cfg.throttle = 0;
	  cfg.soundrate = 0;
	}
	else if(strcasecmp(argv[i], "--nosound") == 0)
	  cfg.soundrate = 0;
	else if(strcasecmp(argv[i], "--sndrate") == 0) {
	  if(++i<argc) cfg.soundrate = atoi(argv[i]);
	}
	else if(strcasecmp(argv[i], "--fullscreen") == 0)
	  cfg.startfullscreen = 1;
	else if(strcasecmp(argv[i], "--fm") == 0)
	  sms.use_fm = 1;
	else if(strcasecmp(argv[i], "--japan") == 0)
	  sms.country = TYPE_DOMESTIC;
	else if(strcasecmp(argv[i], "--usesram") == 0)
	  cfg.use_sram = 1;
  }
}



int main(int argc, char **argv)
{
  int frame_count = 0;
	
  printf("SMS Plus/SDL v%s R%d (Build date %s)\n", VERSION, SDLVERSION, __DATE__);
  printf("(C) Charles Mac Donald in 1998, 1999, 2000\n");
  printf("SDL Version by Gregory Montoir (cyx@frenchkiss.net)\n");
  printf("\n");

  if(argc < 2) {
    printf("Usage: %s <filename.ext> [--options]\n", argv[0]);
    printf("Options:\n");
    printf(" --fskip <n>   \t specify the number of frames to skip.\n");
    printf(" --fullscreen  \t start in fullscreen mode.\n");
    printf(" --fullspeed   \t do not limit updates to 60 frames per second.\n");
    printf(" --nosound     \t disable sound.\n");
    printf(" --sndrate <n> \t specify sound rate (11025, 22050 or 44100).\n");
    printf(" --fm          \t enable YM2413 sound.\n");
    printf(" --japan       \t set the machine type as DOMESTIC instead of OVERSEAS.\n");
    printf(" --usesram     \t load/save SRAM contents before starting/exiting.\n");
    return 1;
  }

  strcpy(cfg.game_name, argv[1]);
  if(load_rom(cfg.game_name) == 0) {
    printf("ERROR: can't load `%s'.\n", cfg.game_name);
    return 1;
  }
  printf("Loaded `%s'.\n", cfg.game_name);

  sms.use_fm = 0;
  sms.country = TYPE_OVERSEAS;
  parse_args(argc, argv);

  printf("Initializing SDL...\n");
  if(!sdl_init_machine(cfg.startfullscreen, cfg.throttle))
	return 1;

  printf("Initializing virtual console emulation...\n");
  system_init(cfg.soundrate);

  if(!snd.enabled || !sdl_init_sound(cfg.soundrate))
	cfg.soundrate = 0;

  if(cfg.use_sram) sms.save = 1;
  load_sram(cfg.game_name);
  
  printf("Starting emulation...\n");
  while(sdl_loop()) {
    int skip_current_frame = 0;

    if(cfg.frame_skip > 1) {
      ++frame_count;
      skip_current_frame = (frame_count % cfg.frame_skip == 0) ? 0 : 1;
    }

    if(!skip_current_frame) {
      sdl_update_display_beginning();
      sms_frame(skip_current_frame);
      sdl_update_display_ending();
    }
    else sms_frame(skip_current_frame);
    
    if(cfg.soundrate) sdl_update_sound_buffer();
   	sdl_sync();
  }

  save_sram(cfg.game_name);
  system_shutdown();
  sdl_trash_machine();
  printf("Leaving...\n");
  return 0;
}
