/*
    Copyright (c) 2002 Gregory Montoir

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <SDL.h>
#include "SDL_framerate.h"
#include "shared.h"
#include "saves.h"
#include "config.h"


#define SMS_SCREEN_WIDTH   256
#define SMS_SCREEN_HEIGHT  192
#define GG_SCREEN_WIDTH    160
#define GG_SCREEN_HEIGHT   144
#define MACHINE_FPS        60
#define WINDOW_TITLE       "SMS+SDL (0.9.3R3)"





static FPSmanager *sdl_fps = NULL;
static SDL_Surface *sdl_screen = NULL;
static SDL_Surface *sdl_bitmap = NULL;
static int used_bitmap_width;
static int used_bitmap_height;



void sdl_update_display_beginning()
{
  SDL_LockSurface(sdl_bitmap);
  bitmap.data = (unsigned char*)sdl_bitmap->pixels;
}



static Uint8 alphatable[256][256];

/* really slow, due to the use of floats... */
SDL_Surface* sdl_create_bilinear_fitered_surface_float(const SDL_Surface *in, int dx, int dy)
{
  SDL_Surface *n;
  /* Create the same surface as the first one (see sdl_init_machine()) - RGB565 */
  n = SDL_CreateRGBSurface(SDL_SWSURFACE, dx, dy, 16, 0xF800, 0x07E0, 0x001F, 0);
  if(n) {
	float fsx = (float)in->w / (float)dx;
	float fsy = (float)in->h / (float)dy;
	float v = 0.0f;
	int j;

	SDL_LockSurface(n);

	for(j = 0; j < dy - 1; ++j) {
	  int i;
	  float u = 0.0f;
	  int iv = (int)v;
      float decy = v - iv;

	  for(i = 0; i < dx; ++i) {
		int iu = (int)u;
		float decx = u - iu;
		float ul = (1.0f - decx) * (1.0f - decy);
		float ur = decx * (1.0f - decy);
		float ll = (1.0f - decx) * decy;
		float lr = decx * decy;

		Uint16 c;
		Uint8 r[5], g[5], b[5];

	    /* we assume the 'in' surface to be locked */
		Uint8* pIn  = (Uint8*)in->pixels + iu * in->format->BytesPerPixel + iv * in->pitch;
		Uint8* pOut = (Uint8*)n->pixels + i * n->format->BytesPerPixel + j * n->pitch;

		c = *(Uint16*)pIn;
		SDL_GetRGB(c, in->format, &r[0], &g[0], &b[0]);
		c = *(Uint16*)(pIn+2);
		SDL_GetRGB(c, in->format, &r[1], &g[1], &b[1]);
		c = *(Uint16*)(pIn+in->pitch);
		SDL_GetRGB(c, in->format, &r[2], &g[2], &b[2]);
		c = *(Uint16*)(pIn+in->pitch+2);
		SDL_GetRGB(c, in->format, &r[3], &g[3], &b[3]);
		r[4] = (Uint8)(ul * r[0] + ur * r[1] + ll * r[2] + lr * r[3]);
		g[4] = (Uint8)(ul * g[0] + ur * g[1] + ll * g[2] + lr * g[3]);
		b[4] = (Uint8)(ul * b[0] + ur * b[1] + ll * b[2] + lr * b[3]);
		*((Uint16*)pOut) = (Uint16)SDL_MapRGB(n->format, r[4], g[4], b[4]);
		u += fsx;
	  }
	  v += fsy;
	  
	}
	/* draw the last line */
	for(; j < dy; ++j) {
	  int i = 0;
	  for(i = 0; i < dx; ++i) {
		Uint8* pIn  = (Uint8*)in->pixels + (Uint8)(i * in->format->BytesPerPixel * fsx) + (Uint8)(j * in->pitch * fsy);
		Uint8* pOut = (Uint8*)n->pixels + i * n->format->BytesPerPixel + j * n->pitch;
		*((Uint16*)pOut) = *((Uint16*)pIn);
	  }
	}
	SDL_UnlockSurface(n);
  }
  return n;
}


/* faster but still too slow... from 12% (float) to 28% (integer) on my C466 */
SDL_Surface* sdl_create_bilinear_fitered_surface_integer(const SDL_Surface *in, int bx, int by, int ix, int iy, int dx, int dy)
{
  SDL_Surface *n;
  n = SDL_CreateRGBSurface(SDL_SWSURFACE, dx, dy, 16, 0xF800, 0x07E0, 0x001F, 0);
  if(n) {
	int scalex = (int)(65536*((float)ix / (float)dx));
	int scaley = (int)(65536*((float)iy / (float)dy));
	Uint32 sx, sy;
	int i, j;

	sx = sy = 0;
	SDL_LockSurface(n);
	for(j = 0; j < dy - 1; ++j) {
	  for(i = 0; i < dx; ++i) {
		Uint8 u = (sx >> 8) & 0xff;
		Uint8 v = (sy >> 8) & 0xff;
/*		Uint8 ul = ((255 - u) * (255 - v)) >> 8;
		Uint8 ur = (u * (255 - v)) >> 8;
		Uint8 ll = ((255 - u) * v) >> 8;
		Uint8 lr = (u * v) >> 8; */
		Uint8 ul = alphatable[255 - u][255 - v];
		Uint8 ur = alphatable[u][255 - v];
		Uint8 ll = alphatable[255 - u][v];
		Uint8 lr = alphatable[u][v];
		Uint16 c;
		Uint8 r[5], g[5], b[5];

	    /* we assume the 'in' surface to be locked */
		Uint8* pIn  = (Uint8*)in->pixels + (bx + (sx >> 16)) * in->format->BytesPerPixel + (by + (sy >> 16)) * in->pitch;
		Uint8* pOut = (Uint8*)n->pixels + i * n->format->BytesPerPixel + j * n->pitch;

		c = *(Uint16*)pIn;
		SDL_GetRGB(c, in->format, &r[0], &g[0], &b[0]);
		c = *(Uint16*)(pIn+2);
		SDL_GetRGB(c, in->format, &r[1], &g[1], &b[1]);
		c = *(Uint16*)(pIn+in->pitch);
		SDL_GetRGB(c, in->format, &r[2], &g[2], &b[2]);
		c = *(Uint16*)(pIn+in->pitch+2);
		SDL_GetRGB(c, in->format, &r[3], &g[3], &b[3]);
		r[4] = ((ul * r[0]) >> 8) + ((ur * r[1]) >> 8) + ((ll * r[2]) >> 8) + ((lr * r[3]) >> 8);
		g[4] = ((ul * g[0]) >> 8) + ((ur * g[1]) >> 8) + ((ll * g[2]) >> 8) + ((lr * g[3]) >> 8);
		b[4] = ((ul * b[0]) >> 8) + ((ur * b[1]) >> 8) + ((ll * b[2]) >> 8) + ((lr * b[3]) >> 8);
		*((Uint16*)pOut) = (Uint16)SDL_MapRGB(n->format, r[4], g[4], b[4]);

		sx += scalex;
	  }
	  sx = 0;
	  sy += scaley;
	}
	/* draw the last line */
	for(; j < dy; ++j) {
	  int i = 0;
	  for(i = 0; i < dx; ++i) {
		Uint8* pIn  = (Uint8*)in->pixels + (Uint8)(i * in->format->BytesPerPixel * (scalex >> 16)) + (Uint8)(j * in->pitch * (scaley >> 16));
		Uint8* pOut = (Uint8*)n->pixels + i * n->format->BytesPerPixel + j * n->pitch;
		*((Uint16*)pOut) = *((Uint16*)pIn);
	  }
	}
	SDL_UnlockSurface(n);
  }
  return n;
}



void sdl_update_display_ending()
{
  if(cfg.use_bilinear_filtering) {
    SDL_Surface *n;
    int beg_x = (IS_GG) ? 48 : 0; /* (SMS_W - GG_W) / 2 */
    int beg_y = (IS_GG) ? 24 : 1; /* (SMS_H - GG_H) / 2 */

    n = sdl_create_bilinear_fitered_surface_integer(sdl_bitmap, beg_x, beg_y, used_bitmap_width, used_bitmap_height, sdl_screen->w, sdl_screen->h);
    SDL_UnlockSurface(sdl_bitmap);
    if(n) {
	  SDL_BlitSurface(n, NULL, sdl_screen, NULL);
	  SDL_UpdateRect(sdl_screen, 0, 0, 0, 0);
	  SDL_FreeSurface(n);
	}
  }
  else {
    SDL_Rect rect_screen;
    int dx = 0;
    int dy = 0;
    /* center the bitmap in the screen if necessary */
    if(sdl_screen->w != sdl_bitmap->w || sdl_screen->h != sdl_bitmap->h) {
      dx = (sdl_screen->w - sdl_bitmap->w) >> 1; 
      dy = (sdl_screen->h - sdl_bitmap->h) >> 1;
      SDL_FillRect(sdl_screen, NULL, 0);
	}
    rect_screen.x = dx;
    rect_screen.y = dy;
    rect_screen.w = sdl_bitmap->w-dx;
    rect_screen.h = sdl_bitmap->h-dy;
    SDL_UnlockSurface(sdl_bitmap);
    SDL_BlitSurface(sdl_bitmap, NULL, sdl_screen, &rect_screen);
    SDL_UpdateRect(sdl_screen, 0, 0, 0, 0);
  }
}


static void sdl_make_screenshot()
{
  char ssname[0x100];
  strcpy(ssname, cfg.game_name);
  sprintf(strrchr(ssname, '.'), "-%03d.bmp", cfg.current_screenshot);
  ++cfg.current_screenshot;
  if(SDL_SaveBMP(sdl_bitmap, ssname) == 0)
	printf("SCREENSHOT to '%s'\n", ssname);
}


static void sdl_init_paddles()
{
  cfg.pad[0].up = SDLK_UP;
  cfg.pad[0].down = SDLK_DOWN;
  cfg.pad[0].left = SDLK_LEFT;
  cfg.pad[0].right = SDLK_RIGHT;
  cfg.pad[0].b1 = SDLK_c;
  cfg.pad[0].b2 = SDLK_v;
  cfg.pad[0].start = SDLK_RETURN;
}


static int sdl_update_input(SDLKey k, int p)
{
  if(!p) {
	switch(k) {
	  case SDLK_F1:
		cfg.state_slot = ++cfg.state_slot % STATE_SLOTS_NB;
		printf("SLOT_CHANGE to %d\n", cfg.state_slot);
		break;
	  case SDLK_F2:
		if(save_state(cfg.game_name, cfg.state_slot))
  		  printf("SAVE_STATE to slot #%d\n", cfg.state_slot);
		break;
	  case SDLK_F3:
		if(load_state(cfg.game_name, cfg.state_slot))
		  printf("LOAD_STATE from slot #%d\n", cfg.state_slot);
		break;
	  case SDLK_F4:
		sdl_make_screenshot();
		break;
	  case SDLK_ESCAPE:
		return 0;
	}
  }
  if(k == cfg.pad[0].start) {
    if(p) input.system |= (IS_GG) ? INPUT_START : INPUT_PAUSE;
    else  input.system &= (IS_GG) ? ~INPUT_START : ~INPUT_PAUSE;
  }
  else if(k == cfg.pad[0].up) {
    if(p) input.pad[0] |= INPUT_UP;
    else  input.pad[0] &= ~INPUT_UP;
  }
  else if(k == cfg.pad[0].down) {
    if(p) input.pad[0] |= INPUT_DOWN;
    else  input.pad[0] &= ~INPUT_DOWN;
  }
  else if(k == cfg.pad[0].left) {
    if(p) input.pad[0] |= INPUT_LEFT;
    else  input.pad[0] &= ~INPUT_LEFT;
  }
  else if(k == cfg.pad[0].right) {
    if(p) input.pad[0] |= INPUT_RIGHT;
    else  input.pad[0] &= ~INPUT_RIGHT;
  }
  else if(k == cfg.pad[0].b1) {
    if(p) input.pad[0] |= INPUT_BUTTON1;
    else  input.pad[0] &= ~INPUT_BUTTON1;
  }
  else if(k == cfg.pad[0].b2) {
    if(p) input.pad[0] |= INPUT_BUTTON2;
    else  input.pad[0] &= ~INPUT_BUTTON2;
  }
  return 1;
}


//#ifdef USE_BILINEAR_FILTERING

static void updateVideo(int w, int h)
{
//  screen_width = w;
//  screen_height = h;
  sdl_screen = SDL_SetVideoMode(w, h, 0, SDL_SWSURFACE|SDL_RESIZABLE);
}

//#endif



int sdl_loop()
{
  SDL_Event event;
  if(SDL_PollEvent(&event)) {
    switch(event.type) {

//#ifdef USE_BILINEAR_FILTERING
	case SDL_VIDEORESIZE:
	  updateVideo(event.resize.w, event.resize.h);
	  return 1;
//#endif

    case SDL_QUIT:
      return 0;
    case SDL_KEYUP:
      return sdl_update_input(event.key.keysym.sym, 0);
    case SDL_KEYDOWN:
      return sdl_update_input(event.key.keysym.sym, 1);
    }
  }
  return 1;
}


static void sdl_calculate_speed()
{
  static Uint32 prev_ticks = 0;
  static int frames_rendered = 0;
  Uint32 cur_ticks = SDL_GetTicks();
  int diff = (cur_ticks - prev_ticks)/1000;
  if(diff < 1)
    ++frames_rendered;
  else {
    char title[100];
    int truefps = frames_rendered / diff;
    int percent = truefps * 100 / MACHINE_FPS;
    sprintf(title,"%s [%d%%]", WINDOW_TITLE, percent);
    SDL_WM_SetCaption(title, NULL);
    frames_rendered = 0;
    prev_ticks = SDL_GetTicks();
  }
}


void sdl_sync()
{
  sdl_calculate_speed();
  /* synchronize if user hasn't disabled it */
  if(sdl_fps) SDL_framerateDelay(sdl_fps);
}



static struct {
  int pos;
  int len;
  Uint8* buf;
} audio;


static void audio_callback(void *userdata, Uint8 *stream, int len)
{
  int i;
  for(i = 0; i < len; ++i) {
    if(audio.pos == audio.len)
	  audio.pos = 0;
    stream[i] = audio.buf[audio.pos++];
  }
}


int sdl_init_sound(int samplerate)
{
  SDL_AudioSpec as;
  if(SDL_InitSubSystem(SDL_INIT_AUDIO) < 0) {
    printf("ERROR: can't initialize SDL AUDIO SUBSYSTEM: %s.\n", SDL_GetError());
    return 0;
  }
  as.freq = samplerate;
  as.format = AUDIO_S16;
  as.channels = 2;
  as.samples = 4096;
  as.callback = audio_callback;
  as.userdata = 0;
  if(SDL_OpenAudio(&as, NULL) == -1) {
    printf("ERROR: can't open audio: %s.\n", SDL_GetError());
    return 0;
  }
  audio.buf = (Uint8*)malloc(as.size);
  if(!audio.buf) {
    printf("ERROR: can't allocate memory for sound.\n");
    return 0;
  }
  audio.len = as.size;
  audio.pos = 0;
  memset(audio.buf, 0, audio.len);
  cfg.soundrate = as.freq;
 
  printf("Initializing SDL Sound... %dHz 16bits %s.\n", as.freq, as.channels==2?"stereo":"mono");
  SDL_PauseAudio(0);
  return 1;
}


void sdl_update_sound_buffer()
{
  int i;

  SDL_LockAudio();
  for(i = 0; i < snd.bufsize; ++i) {
	audio.pos = (audio.pos + 2) % audio.len; 
	*((short*)(audio.buf + audio.pos)) = snd.buffer[0][i];
  }
  for(i = 0; i < snd.bufsize; ++i) {
	audio.pos = (audio.pos + 2) % audio.len;
	*((short*)(audio.buf + audio.pos)) = snd.buffer[1][i];
  }
  SDL_UnlockAudio();
}



int sdl_init_machine(int fullscreen, int throttle)
{
  int screen_width, screen_height;
  Uint32 vidflags = SDL_SWSURFACE|SDL_RESIZABLE;

  /* to speed up the bilinear filtering routine */
  int i, j;
  for(j = 0; j < 256; ++j)
	for(i = 0; i < 256; ++i)
	  alphatable[i][j] = (i * j) >> 8;

  screen_width  = used_bitmap_width  = (IS_GG) ? GG_SCREEN_WIDTH  : SMS_SCREEN_WIDTH;
  screen_height = used_bitmap_height = (IS_GG) ? GG_SCREEN_HEIGHT : SMS_SCREEN_HEIGHT;

  if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0) {
    printf("ERROR: can't initialize SDL VIDEO|TIMER SUBSYSTEM: %s.\n", SDL_GetError());
    return 0;
  }

  if(fullscreen) {
    vidflags |= SDL_FULLSCREEN;
    screen_width = 320;
    screen_height = 200;
    SDL_ShowCursor(SDL_DISABLE);
  }
  sdl_screen = SDL_SetVideoMode(screen_width, screen_height, 0, vidflags);
  if(!sdl_screen) {
    printf("ERROR: can't set video mode (%dx%d): %s.\n", screen_width, screen_height, SDL_GetError());
    return 0;  	
  }
  
  /* Create the 16 bits bitmap surface (RGB 565) */
  /* Even if we are in GameGear mode, we need to allocate space for a buffer of 256x192 */
  sdl_bitmap = SDL_CreateRGBSurface(SDL_SWSURFACE, SMS_SCREEN_WIDTH, SMS_SCREEN_HEIGHT, 16, 0xF800, 0x07E0, 0x001F, 0);
  if(!sdl_bitmap) {
    printf("ERROR: can't create primary surface: %s.\n", SDL_GetError());
    return 0;  	  	
  }

  memset(&bitmap, 0, sizeof(t_bitmap));
  bitmap.width  = sdl_bitmap->w;
  bitmap.height = sdl_bitmap->h;
  bitmap.depth  = 16;
  bitmap.pitch  = sdl_bitmap->pitch;
  bitmap.data   = (unsigned char *)sdl_bitmap->pixels;

  SDL_WM_SetCaption(WINDOW_TITLE, NULL);

  sdl_init_paddles();
  SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

  if(throttle) {
	sdl_fps = (FPSmanager*)malloc(sizeof(FPSmanager));
	if(sdl_fps) {
	  SDL_initFramerate(sdl_fps);
	  SDL_setFramerate(sdl_fps, MACHINE_FPS);
	}
  }
  return 1;
}


void sdl_trash_machine()
{
  if(cfg.soundrate != 0) { 
	SDL_CloseAudio();
	free(audio.buf);
  }
  if(sdl_screen) SDL_FreeSurface(sdl_screen);
  if(sdl_bitmap) SDL_FreeSurface(sdl_bitmap);
  if(sdl_fps) free(sdl_fps);
  SDL_Quit();
}
