
/* Ripped and modified from the original DOS port */

#include "defines.h"
#ifdef SMSSDL_SEALSOUND
#include <audio.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include "sealintf.h"



#define MAX_SOUND               4
#define NUMVOICES               16
#define MAX_STREAM_CHANNELS     6



int seal_sample_rate =   44100;

HAC hVoice[NUMVOICES];
LPAUDIOWAVE lpWave[NUMVOICES];
AUDIOINFO info;
AUDIOCAPS caps;
int c[MAX_STREAM_CHANNELS];
int nominal_sample_rate;
static int attenuation = 0;
static int master_volume = 256;


void osd_update_audio(void)
{
    if (seal_sample_rate == 0) return;
    AUpdateAudio();
}


/* attenuation in dB */
void osd_set_mastervolume(int _attenuation)
{
    float volume;

    attenuation = _attenuation;

    volume = 256.0; /* range is 0-256 */
    while (_attenuation++ < 0)
        volume /= (float)1.122018454;  /* = (10 ^ (1/20)) = 1dB */

    master_volume = (int)volume;

    ASetAudioMixerValue(AUDIO_MIXER_MASTER_VOLUME,master_volume);
}


int seal_sound_init(int rate)
{
    int i;

    seal_sample_rate = rate;

    if (AInitialize() != AUDIO_ERROR_NONE)
        return 0;

    /* open audio device */
    info.nDeviceId = AUDIO_DEVICE_MAPPER;
    /* always use 16 bit mixing if possible - better quality and same speed of 8 bit */
    info.wFormat = AUDIO_FORMAT_16BITS | AUDIO_FORMAT_STEREO; // | AUDIO_FORMAT_RAW_SAMPLE;

    info.nSampleRate = seal_sample_rate;
    if (AOpenAudio(&info) != AUDIO_ERROR_NONE)
        return 0;

    AGetAudioDevCaps(info.nDeviceId,&caps);
	if(!(info.wFormat & AUDIO_FORMAT_16BITS) || !(info.wFormat & AUDIO_FORMAT_STEREO) || info.nSampleRate != rate) {
      printf("ERROR: soundcard driver does not accept specified sound format.\n");
	  return 0;
	}

    /* open and allocate voices, allocate waveforms */
    if (AOpenVoices(NUMVOICES) != AUDIO_ERROR_NONE)
    {
        printf("ERROR: voices initialization failed\n");
        return 1;
    }

    for (i = 0; i < NUMVOICES; i++)
    {
        if (ACreateAudioVoice(&hVoice[i]) != AUDIO_ERROR_NONE)
        {
            printf("ERROR: voice #%d creation failed\n",i);
            return 1;
        }

        ASetVoicePanning(hVoice[i],128);

        lpWave[i] = 0;
    }

    /* update the Machine structure to reflect the actual sample rate */
    seal_sample_rate = info.nSampleRate;

    {
        clock_t a,b;
        LONG start,end;

        if ((lpWave[0] = (LPAUDIOWAVE)malloc(sizeof(AUDIOWAVE))) == 0)
            return 1;

        lpWave[0]->wFormat = AUDIO_FORMAT_8BITS | AUDIO_FORMAT_MONO;
        lpWave[0]->nSampleRate = seal_sample_rate;
        lpWave[0]->dwLength = 3*seal_sample_rate;
        lpWave[0]->dwLoopStart = 0;
        lpWave[0]->dwLoopEnd = 3*seal_sample_rate;
        if (ACreateAudioData(lpWave[0]) != AUDIO_ERROR_NONE)
        {
            free(lpWave[0]);
            lpWave[0] = 0;

            return 0;
        }

        memset(lpWave[0]->lpData,0,3*seal_sample_rate);
        /* upload the data to the audio DRAM local memory */
        AWriteAudioData(lpWave[0],0,3*seal_sample_rate);
        APrimeVoice(hVoice[0],lpWave[0]);
        ASetVoiceFrequency(hVoice[0],seal_sample_rate);
        ASetVoiceVolume(hVoice[0],0);
        AStartVoice(hVoice[0]);

        a = clock();
        /* wait some time to let everything stabilize */
        do
        {
            osd_update_audio();
            b = clock();
        } while (b-a < CLOCKS_PER_SEC/10);

        a = clock();
        AGetVoicePosition(hVoice[0],&start);
        do
        {
            osd_update_audio();
            b = clock();
        } while (b-a < CLOCKS_PER_SEC);
        AGetVoicePosition(hVoice[0],&end);

        nominal_sample_rate = seal_sample_rate;
        seal_sample_rate = end - start;

        AStopVoice(hVoice[0]);
        ADestroyAudioData(lpWave[0]);
        free(lpWave[0]);
        lpWave[0] = 0;
    }

    osd_set_mastervolume(0);    /* start at maximum volume */

    return 1;
}


void seal_sound_close(void)
{
    if (seal_sample_rate != 0)
    {
        int n;

        /* stop and release voices */
        for (n = 0; n < NUMVOICES; n++)
        {
            AStopVoice(hVoice[n]);
            ADestroyAudioVoice(hVoice[n]);
            if (lpWave[n])
            {
                ADestroyAudioData(lpWave[n]);
                free(lpWave[n]);
                lpWave[n] = 0;
            }
        }
        ACloseVoices();
        ACloseAudio();
    }
}


void playstreamedsample(int channel,signed char *data,int len,int freq,int volume,int pan,int bits)
{
    static int playing[NUMVOICES];
    static int c[NUMVOICES];

    /* backwards compatibility with old 0-255 volume range */
    if (volume > 100) volume = volume * 25 / 255;

    if (seal_sample_rate == 0 || channel >= NUMVOICES) return;

    if (!playing[channel])
    {
        if (lpWave[channel])
        {
            AStopVoice(hVoice[channel]);
            ADestroyAudioData(lpWave[channel]);
            free(lpWave[channel]);
            lpWave[channel] = 0;
        }

        if ((lpWave[channel] = (LPAUDIOWAVE)malloc(sizeof(AUDIOWAVE))) == 0)
            return;

        lpWave[channel]->wFormat = (bits == 8 ? AUDIO_FORMAT_8BITS : AUDIO_FORMAT_16BITS)
                | AUDIO_FORMAT_MONO | AUDIO_FORMAT_LOOP;
        lpWave[channel]->nSampleRate = nominal_sample_rate;
        lpWave[channel]->dwLength = 3*len;
        lpWave[channel]->dwLoopStart = 0;
        lpWave[channel]->dwLoopEnd = 3*len;
        if (ACreateAudioData(lpWave[channel]) != AUDIO_ERROR_NONE)
        {
            free(lpWave[channel]);
            lpWave[channel] = 0;

            return;
        }

        memset(lpWave[channel]->lpData,0,3*len);
        memcpy(lpWave[channel]->lpData,data,len);
        /* upload the data to the audio DRAM local memory */
        AWriteAudioData(lpWave[channel],0,3*len);
        APrimeVoice(hVoice[channel],lpWave[channel]);
    /* need to cast to double because freq*nominal_sample_rate can exceed the size of an int */
        ASetVoiceFrequency(hVoice[channel],
			(double)freq*nominal_sample_rate/seal_sample_rate);
        AStartVoice(hVoice[channel]);
        playing[channel] = 1;
        c[channel] = 1;
    }
    else
    {
        LONG pos;

        for(;;)
        {
                AGetVoicePosition(hVoice[channel],&pos);
                if (c[channel] == 0 && pos >= len) break;
                if (c[channel] == 1 && (pos < len || pos >= 2*len)) break;
                if (c[channel] == 2 && pos < 2*len) break;
                osd_update_audio();
        }

        memcpy(&lpWave[channel]->lpData[len * c[channel]],data,len);
        AWriteAudioData(lpWave[channel],len*c[channel],len);
        c[channel]++;
        if (c[channel] == 3) c[channel] = 0;
    }


    ASetVoiceVolume(hVoice[channel],volume * 64 / 100);
    ASetVoicePanning(hVoice[channel],(pan + 100) * 255 / 200);
}

void osd_play_streamed_sample_16(int channel,signed short *data,int len,int freq,int volume,int pan)
{
    playstreamedsample(channel,(signed char *)data,len,freq,volume,pan,16);
}


#endif