
/*
    Copyright (c) 2002 Gregory Montoir

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include "defines.h"
#include "sdlsms.h"
#include "saves.h"
#include "shared.h"
#include "config.h"



config cfg = {
  FILTER_NORMAL, /* normal rendering */
  0, /* current screenshot file (*-000.bmp) */
  0, /* no sound core */
  0, /* default sound rate */
  1, /* throttle enabled */
  0, /* state_slot to use */
  1, /* 1 for no frame skip */
  0, /* start windowed (not fullscreen) */
  0, /* do not use sram (save, load) before starting/exiting */
  "",
  -1, /* no joystick */
  {1, 2, 4} /* default joystick mapping */
};



static int parse_args(int argc, char **argv)
{
  int i;

  /* default virtual console emulation settings */
  sms.use_fm = 0;
  sms.country = TYPE_OVERSEAS;

  strcpy(cfg.game_name, argv[1]);
  if(load_rom(cfg.game_name) == 0) {
    printf("ERROR: can't load `%s'.\n", cfg.game_name);
    return 0;
  }
  else printf("Loaded `%s'.\n", cfg.game_name);

  for(i = 2; i < argc; ++i) {
    if(strcasecmp(argv[i], "--fskip") == 0) {
      if(++i<argc) {
        cfg.frame_skip = atoi(argv[i]);
        cfg.throttle = 0;
      }
    }
    else if(strcasecmp(argv[i], "--fullspeed") == 0) {
      cfg.throttle = 0;
      cfg.sound_rate = 0;
    }
    else if(strcasecmp(argv[i], "--fm") == 0)
      sms.use_fm = 1;
    else if(strcasecmp(argv[i], "--japan") == 0)
      sms.country = TYPE_DOMESTIC;
    else if(strcasecmp(argv[i], "--usesram") == 0) {
      cfg.use_sram = 1;
      sms.save = 1;
    }
    else if(strcasecmp(argv[i], "--fullscreen") == 0)
      cfg.start_fullscreen = 1;
    else if(strcasecmp(argv[i], "--joystick") == 0) {
      i++;
      if(i < argc)
        cfg.joystick_number = atoi(argv[i]);
    }
    else if(strcasecmp(argv[i], "--filter") == 0) {
      i++;
      if(i < argc) {
        switch(atoi(argv[i])) {
        case 1: cfg.render_mode = FILTER_BILINEAR;
          break;
        case 2: cfg.render_mode = FILTER_SCALE2XADVMAME;
          break;
        case 3: cfg.render_mode = FILTER_2XSAI;
          break;
        case 4: cfg.render_mode = FILTER_SUPER2XSAI;
          break;
        case 5: cfg.render_mode = FILTER_SUPEREAGLE;
          break;
        case 6: cfg.render_mode = FILTER_SCANLINES50;
          break;
        case 7: cfg.render_mode = FILTER_SCANLINES100;
          break;
#if USE_MMX
        case 8: cfg.render_mode = FILTER_SCANLINES50MMX;
          break;
#endif
        default:
          printf("WARNING: unknown rendering filter '%s'.\n", argv[i]);
        }
      }
    }
    else if(strcasecmp(argv[i], "--sound") == 0) {
      cfg.sound_rate = 22050;
      i++;
      if(i < argc) {
        switch(atoi(argv[i])) {
        case 1: cfg.audio_core = AUDIO_SDL;
          break;
#if USE_WIN32
        case 2: cfg.audio_core = AUDIO_WIN32API;
          break;
#endif
        default:
          printf("WARNING: unknown audio core '%s'.\n", argv[i]);
          cfg.sound_rate = 0;
        }
      }
    }
    else 
      printf("WARNING: unknown option '%s'.\n", argv[i]);
  }
  return 1;
}



int main(int argc, char **argv)
{
  printf("%s (Build date %s)\n", SMSSDL_TITLE, __DATE__);
  printf("(C) Charles Mac Donald in 1998, 1999, 2000\n");
  printf("SDL Version by Gregory Montoir (cyx@frenchkiss.net)\n");
  printf("\n");

  if(argc < 2) {
    printf("Usage: %s <filename.<SMS|GG>> [--options]\n", argv[0]);
    printf("Options:\n");
    printf(" --fskip <n>      \t specify the number of frames to skip.\n");
    printf(" --fullspeed      \t do not limit updates to 60 frames per second.\n");
    printf(" --fm             \t enable YM2413 sound.\n");
    printf(" --japan          \t set the machine type as DOMESTIC instead of OVERSEAS.\n");
    printf(" --usesram        \t load/save SRAM contents before starting/exiting.\n");
    printf(" --fullscreen     \t start in fullscreen mode.\n");
    printf(" --printdefines   \t print compilation defines.\n");
    printf(" --joystick <n>   \t use a joystick.\n");
    printf(" --filter <mode>  \t render using a filter:\n");
    printf("                  \t 1 -> bilinear filtering effect.\n");
    printf("                  \t 2 -> AdvanceMAME Scale2x effect.\n");
    printf("                  \t 3 -> 2x SaI effect.\n");
    printf("                  \t 4 -> Super 2x SaI effect.\n");
    printf("                  \t 5 -> Super Eagle effect.\n");
    printf("                  \t 6 -> scanlines 50%% effect.\n");
    printf("                  \t 7 -> scanlines 100%% effect.\n");
#if USE_MMX
    printf("                  \t 8 -> scanlines 50%% MMX effect.\n");
#endif
    printf(" --sound <core>   \t enable sound (22050Hz):\n");
    printf("                  \t 1 -> use SDL internal sound subsystem.\n");
#if USE_WIN32
    printf("                  \t 2 -> use WIN32 API.\n");
#endif
    return 1;
  }

  if(!parse_args(argc, argv))
	return 0;

  if(!sdl_main_init())
	return 0;

  sdl_main_loop();

  sdl_main_close();

  return 0;
}
